/*
============================================================================================
CONTEXT:
This program computes the numerical results that are reported in the paper
"Auctions and Corruption: An Analysis of Ex Post Bid Rigging" by Lengwiler and Wolfstetter.

AUTHORS:
Yvan Lengwiler, University of Basel, Switzerland
Elmar Wolfstetter, Humboldt University, Berlin, Germany

DATE:
August 2007 (current version)

DESCRIPTION:
The program computes an approximate symmetric equilibrium of the first-price auction with
corruption. We allow for type I and type II corruption. There are n bidders with uniformly
distributed valuations. Please consult the paper for further details.

OUTPUT:
The program dumps the result onto the standard output (a command line window). Make sure
that the buffer of this window contains at least 145 columns (to avoid line wraps) and
sufficiently many rows (depending on the parameters you choose) to avoid loss of data.
One can also redirect the output into a file with the command line option "... > output.txt"

TECHNICAL NOTES:
The program is written in C# and assumes the Microsoft .NET Framework SDK Version 1.1 to
be present on the computer. Download .NET from http://msdn.microsoft.com/netframework/ in
case you have not done so yet.
We originally used the SharpDevelop IDE for developing this program, but later moved to the
Microsoft Visual C# 2005 Express Edition. SharpDevelop can be download for free from
http://www.icsharpcode.net/OpenSource/SD/. MS C# Express can be downloaded from the
Microsoft website, http://msdn.microsoft.com/vstudio/express/visualcsharp/default.aspx.
============================================================================================
*/

using System;
namespace BidRigging
{
	class Computation
	{
		// =====================================================================================
		// In this section, the parameters of the problem are given.
		// You are free to change these parameters.
		
		const byte n = 2;		        // # bidders (greater than or equal to 2)
		const double alpha = 0.7;	    // the bidder's share (between 0.0 and 1.0)
//		const double alpha = 0.6101;	// borderline alpha for n = 2
//		const double alpha = 0.5010;	// borderline alpha for n = 3
//		const double alpha = 0.4515;	// borderline alpha for n = 5
//		const double alpha = 0.4013;	// borderline alpha for n = 10
		
		// =====================================================================================
		// Next we define the parameters defining the discrete approximation, the methods
		// for find the root and the initial guess, and the stopping conditions.
		// These settings should only be changed with caution.
		
		// --- discrete valuation space --------------------------------------------------------
		
		static int g = 200;		    	        // initial # of points in the space of valuations
		const int SubdivisionFactor = 3;		// factor by which valuation space is subdivided
		const int NbSubdivisions = 0;			// number of times the space is subdivided
		
		// "delta" for finite difference derivatives
		const double diffDelta = 1E-10;			// this is the "delta" we use to compute the
												// derivatives
		// --- initital bid function -----------------------------------------------------------
		
		const int initmethod = 0;			    // choice of method for initial guess
												// initmethod = 0 : linear guess
												// initmethod = 1 : grid search

		const double slope = (n-1)/(n-alpha);	// slope of linear initial guess

		const int GridSearchSteps = 50;			// parameter for the grid search
												// GridSearchSteps = # values that are tried for
												// each point in the valuation grid
		
		// --- step size optimization ----------------------------------------------------------
		
		const int taumethod = 1;		        // taumethod = 0: brute force
							                    // taumethod = 1: golden section search

		const int tausteps = 10;		        // number of stepsizes we try out in each iteration step
							                    // when using brute force
		const double tauEps = 1E-2;		        // required precision when using golden section search

		const double GRANDMAXTAU = 2.0;	        // maximum step size under all circumstances

		// --- root finding --------------------------------------------------------------------
		
		const int rootmethod = 3;		        // choice of method for root finding
                    							// rootmethod = 0 : steepest descent method
					                    		// rootmethod = 1 : Gauss-Newton method (inverse Jacobian)
							                    // rootmethod = 2 : hybrid method
							                    // rootmethod = 3 : Levenberg-Marquardt method
		
		// switching rule for hybrid method
		const double switchEps = 1E-12;
		const double switchKeep = 5;
		
		// coefficients for Levenberg-Marquardt method
		const double initMu = 1E-3;		        // initial mu
		const double muFactor = 10;		        // factor by which mu is multiplied or divided
		const double maxMu = 1E+100;		    // upper bound for mu
		const double minMu = Double.Epsilon;	// lower bound for mu
		
		// --- stopping rules ------------------------------------------------------------------
		
		const double eps = Double.Epsilon;	    // stop iterating if SSE < eps
							                    // (Double.Epsilon is machine precision, so this
							                    // effectively turns off this stopping rule; you
							                    // can choose a larger value for eps)
		const int keepiter = 20;		        // stop iterating if no improvement in so many steps
		const int maxiter = 100;		        // stop after so many steps in any case
		
		// =====================================================================================
		// Next are declarations of variables that are used by the program.
		// Do not make any changes below this line unless you know what you are doing.
		
		// ultimate fineness of the grid on the valuation space
		static int max_g = g * (int) Math.Pow(SubdivisionFactor,NbSubdivisions);
		
		// The bid function (beta) and the two limits of integration (phi and psi) are computed
		// on the grid of valuations {0,...,g} and stored in these arrays.
		
		static double[,] betavec = new double[max_g+1, 3];
		static double[,] phivec = new double[max_g+1, 3];
		static double[,] psibarvec = new double[max_g+1, 3];

        // The payoff function U(v,x) is stored in a matrix with rows x columns {0,...,g} x {0,...,g}.
        static double[,] Uvec = new double[max_g + 1, max_g + 1];
        static double[,] psitildevec = new double[max_g + 1, max_g + 1];
        static double[,] psivec = new double[max_g + 1, max_g + 1];
        // themax(v) = max_x U(v,x), argmax(v) = argmax_x U(v,x)
        // The equilibrium condition is that v = argmax(v).
        static double[] themax = new double[max_g + 1];
        static double[] argmax = new double[max_g + 1];

        // The same is true for the derivatives. betaPrime[j] is the slope of beta for v in the
		// interval between the (j-1)'th and the j'th grid point (so betaPrime[0,k] is not
		// used). phiPrime[j] and psibarPrime[j] are the derivatives of phi and psi, respecively,
		// at the j'th grid point.
		
		static double[,] betaPrime = new double[max_g+1, 3];
		static double[,] phiPrime = new double[max_g+1, 3];
		static double[,] psibarPrime = new double[max_g+1, 3];
		
		// variables for welfare analysis
		
		static double[] H = new double[max_g+1];
		static double[] IntH = new double[max_g+1];
		static double Welfare = 0;
		static double WelfareLoss = 0;
		static double WelfareEfficient = 0;
		static double piAuctioneer = 0;
		static double piSeller = 0;
		static double piAllBidders = 0;
		static double[] Zvec = new double[max_g+1];
		
		// The differential equation is also evaluated on all the grid points.
		// SSE is the root mean squared error.
		// maxabserr is the maximum absolute value of it on all the grid points.
		// meanerr is the average value of the diff eq over all the grid points.
		// grad is the gradient of SSE with respect to betavec.
		
		static double[,] D = new double[max_g+1, 3];
		static double[] SSE = new double[3];
		static double[] grad = new double[max_g+1];
		
		// We keep track of the errors in order to pick the best iteration step.
		static double[] bestBetavec = new double[max_g+1];
		static int bestIter = 0;
		static double bestSSE = 0;
		
		// Variables for storing a gxg matrix and its inverse
		static double [,] J = new double[max_g+1, max_g+1];		    // The Jacobian ...
		static double [,] invJ = new double[max_g+1, max_g+1];		// ... and its inverse
		static double [,] TransJ = new double[max_g+1, max_g+1];	// This holds the transpose
		static double [,] keepJ = new double[max_g+1, max_g+1];		// occasionally we also need
										                            // a copy of the Jacobian

        // variables for finding the best stepsize; used in OptimizeTau()
		static double[] taubar = new double[max_g+1];		// the largest stepsize that satisfies ...
		static double TAUBAR;					            // ... weak shading
		static double[] tautilde = new double[max_g+1];		// the largest stepsize that satisfies ...
		static double TAUTILDE;					            // ... weak monotonicity
		static double TAUSTAR;					            // the stronger of the two constraints
		static double golden = (Math.Sqrt(5)-1)/2;		    // factor for golden section
		
		static double[] factor = new double[tausteps];		// element of {1/tausteps, ..., 1-1/tausteps}
		static double tau = 0;					            // stepsize
		static double besttau = 0;				            // best stepsite
		static double[] SSEtau = new double[tausteps];		// SSE at stepsize tau
		static double bestSSEtau = 0;				        // SSE at best stepsize
		
		static double shrinkTimes = 0;
		static int bestFactor = 1;
		
		// switch that determines method to be used in current iteration step
		static int currentmethod;							// method for current iteration step
		
		// variable for Levenberg-Marquardt method
		static double mu = initMu;
		
		// counter for the grid search
		static Int64 GridCount = 0;
		static Int64 MaxGridCount = (Int64) Math.Pow(GridSearchSteps,g);
		static double GridPercent = 0;
		
		// k is a global identifier with value 0, 1, or 2.
		// k = 0 is the main copy of the vectors;
		// k = 1,2 are auxiliary copies we use to compute (approximate) derivatives.
		
		static byte k = 0;
		
		
		// ==================================================================================
		
		public static void Main()
		{
			// This is the entry point of the program.
			
			int SubdivCounter = 0;	// counter for subdivisions of valiuation space
			int j = 0;		// index on the grid
			
			// initialize 'factor[]'
			//   factor[] is used only in OptimizeTau and it would make sense to initialize
			//   it there (and actually to define it as a local variable in OptimizeTau),
			//   but it is more efficient to declare this variable globally and initialize
			//   it only once, so this is what we do.
			for (j=1; j<=tausteps-1; j++)
				factor[j] = (double) j/tausteps;
			
			// report the parameters of the calculation
			OutputParams();
			
			// use a given initial linear bid function;
			// alternatively, do grid search to find a good initial guess for the root finding procedure
			switch (initmethod)
			{
				case 0:
					InitLinear();
					break;
				case 1:
					RunGridSearch();
					break;
			}
			
			// report the bid functions we start from for the root finding procedure
			// (do not report a simple linear bid function)
			if (!(initmethod == 0)) OutputResult();
			
			switch (rootmethod)
			{
				case 0:
					Console.WriteLine(">>> Using steepest descent method...\n");
					break;
				case 1:
					Console.WriteLine(">>> Using Gauss-Newton method...\n");
					break;
				case 2:
					Console.WriteLine(">>> Using hybrid method...\n");
					Console.WriteLine(">>> starting with steepest descent method...");
					break;
				case 3:
					Console.WriteLine(">>> Using Levenberg-Marquardt method...\n");
					break;
			}
			
			// this is the loop that makes the grid progressively finer
			for (SubdivCounter=0; SubdivCounter<=NbSubdivisions; SubdivCounter++)
			{
				// compute approximate solution
				FindRoot();
				
				// report result for this grid
				Console.Write("\n\nbest iteration is # {0}", bestIter);
				OutputResult();
				
				// subdivide the grid
				if (SubdivCounter < NbSubdivisions) SubdivideValuationGrid();
			}

            // compute and print out welfare and distribution implications
			ComputeH();		        // compute prob of winning
			ComputePayoffs();	    // compute expected payoffs of auctioneer and seller
						            // and ex-ante efficiency loss
			OutputWelfareResults();	// report results

            // compute off-equilibrium payoffs and test for global optimality
            ComputePsitilde();          // compute constraints in proposal-, acceptance, rejection-regions
            ComputeUvec();              // compute U(v,x)
            OutputEquilibriumTest();    // check is v = argmax_x U(v,x)

            OutputPsiAndU(0.25);    // output psitilde, U, etc.
            OutputPsiAndU(0.5);     // output psitilde, U, etc.
            OutputPsiAndU(0.75);    // output psitilde, U, etc.

//            OutputPsiAndU(0.485);    // output psitilde, U, etc.

            Console.WriteLine("Press 'x' and 'enter' to exit...");
			wait('x');
		}

        private static void wait(char resume_char)
		{
			char c;
			do c = (char) Console.Read();
			while (!(c == resume_char));
		}
		
		private static void SubdivideValuationGrid()
		{
			int j = 0; int jj = 0;
			for (j=0; j<g; j++)
			{
				for (jj=1; jj<=SubdivisionFactor; jj++)
				{
					betavec[SubdivisionFactor*j+jj,1] = betavec[j,0] + betaPrime[j+1,0] * (double) jj / (g * SubdivisionFactor);
				}
			}
			g *= SubdivisionFactor;
			copyVectors(1,0);
			k = 0;
			compute();
		}

        // ****************************************
		// INITIAL BID FUNCTION
		// ****************************************
		
		private static void InitLinear()
		{
			// Here we initialize the vectors with a linear bid function
			// (with given slope). The iteration will start from here.
			int j = 0;	// index on the grid
			
			Console.WriteLine(">>> Filling in linear bid function as an initial guess...");
			Console.WriteLine("    (slope = {0})\n", slope);
			
			// fill vectors with initial "guess"
			k = 0;
			for (j=1; j<=g; j++)
				betavec[j, k] = slope * val(j);
			compute();
		}
		
		private static void RunGridSearch()
		{
			// We run a grid search to find a good starting point for the root finding procedure.
			// For each of the g valuation, we select G points, all satisfying weak shading and
			// strict monotonicity. This gives rise to G^g bid functions. We evaluate the SSE of
			// all these bid functions ("by brute force") and select the one with the smallest
			// SSE as our initial guess.
			
			int j = 0;
			
			Console.WriteLine(">>> Running grid search to find initial guess...\n");
			
			// start from the smallest bid function we are searching through
			for (j=1; j<=g; j++)
				betavec[j,0] = (double)1/GridSearchSteps*(val(j)-betavec[j-1,0]);
			// initialize bestSSE and bestBetavec
			compute();
			bestSSE = SSE[0];
			for (j=0; j<=g; j++)
				bestBetavec[j] = betavec[j, 0];
			// now start the grid search
			GridSearch(1);
			// restore best estimate
			for (j=0; j<=g; j++)
				betavec[j, 0] = bestBetavec[j];
			Console.Write("\n");
		}
					
		private static void GridSearch(int dimen)
		{
			// this is part of the grid search algorithm
			// note: this routine is called recursively
			
			int i = 0; int j = 0;		// two indices
			
			if (dimen == g)
			{
				for (i=1; i<=GridSearchSteps; i++)
				{
					betavec[g,0] = (double)i/GridSearchSteps*(val(g)-betavec[g-1,0]);
					betavec[g,0] += betavec[g-1,0];
					compute();
					// check if current SSE is best so far; if so, store current result
					if (SSE[0] < bestSSE)
					{
						bestSSE = SSE[0];
						for (j=0; j<=g; j++)
							bestBetavec[j] = betavec[j, 0];
					}
					GridCount++;
					GridPercent = (double) GridCount/MaxGridCount * 100;
					Console.Write("best SSE = {0}, current SSE = {1} ({2}%)\r",
					    bestSSE.ToString("0.00E+0"), SSE[0].ToString("0.00E+0"), GridPercent.ToString("0.0"));
				}
			}
			else
			{
				for (i=1; i<=GridSearchSteps; i++)
				{
					betavec[dimen,0] = (double)i/GridSearchSteps*(val(dimen)-betavec[dimen-1,0]);
					betavec[dimen,0] += betavec[dimen-1,0];
					GridSearch(dimen+1);
				}
			}
		}
		
		// ****************************************
		// ROOT FINDING PROCEDURES
		// ****************************************
		
		private static void FindRoot()
		{
			// computation of approximate beta with the the steepest descent method
			
			int iter = 0;		// iterations counter
			int j = 0;		    // a counters
			
			mu = initMu;		// reset mu for Levenberg-Marquardt method
			
			int lastSignificantImprovementIter;
			double lastSignificantImprovementSSE;
			
			// initialize "currentmethod" switch
			if (rootmethod == 2)		// hybrid method
				currentmethod = 0;	    // start with steepest descent method
			else
				currentmethod = rootmethod;

			// report statistics of starting point
			Console.WriteLine ("current grid: {0}, initial SSE: {1}", g, SSE[0].ToString("0.00E+0"));
				
			// initialize "best iteration" with initial bid function
			bestIter = 0;
			bestSSE = SSE[0];

			lastSignificantImprovementIter = 0;
			lastSignificantImprovementSSE = SSE[0];

			for (j=0; j<=g; j++) bestBetavec[j] = betavec[j, 0];
			
			// start iteration over betavec
			iter = 0;
			TAUSTAR = 1;
			while (iter < bestIter + keepiter && bestSSE > eps && iter < maxiter && TAUSTAR > 0)
			{
				iter++;
				
				// perform iteration step
				switch (currentmethod)
				{
					case 0:
						SteepestDescentStep();
						break;
					case 1:
						GaussNewtonStep();
						break;
					case 3:
						LevenbergMarquardtStep();
						break;
				}
				
  				// find the best stepsize
  				if (taumethod == 0)
  					OptimizeTau_BruteForce();
				else
			        OptimizeTau_GoldenSection();
				
				// check if current SSE is best so far; if so, store current result
				if (SSE[0] < bestSSE)
				{
					// check if improvement is "significant" (for hybrid method)
					if (lastSignificantImprovementSSE - bestSSE > switchEps)
					{
						lastSignificantImprovementIter = iter;
						lastSignificantImprovementSSE = SSE[0];
					}
					// store new bestBetavec iteration
					bestIter = iter;
					bestSSE = SSE[0];
					for (j=0; j<=g; j++)
						bestBetavec[j] = betavec[j, 0];
				}
				
				// report current result
				if (currentmethod == 3)		// if method is Levenberg-Marquardt
				{
					Console.Write ("best iter: {0} (SSE: {1}), current iter: {2} (SSE: {3}), mu: {4}, tau*: {5}, tau: {6}, no imprvmnt since {7} iters.   \r",
					                  bestIter, bestSSE.ToString("0.00E+0"), iter, SSE[0].ToString("0.00E+0"), mu.ToString("0.0E+0"), TAUSTAR.ToString("0.00E+0"),
					                  besttau.ToString("0.00E+0"), iter - bestIter);
				}
				else
				{
					if (rootmethod == 2 && currentmethod == 0)	// if method is hybrid and current method is steepest descent
					{
						Console.Write ("best iter: {0} (SSE: {1}), current iter: {2} (SSE: {3}), tau*: {4}, tau: {5}, no imprvmnt since {6} ({7}) iters.   \r",
						                  bestIter, bestSSE.ToString("0.00E+0"), iter, SSE[0].ToString("0.00E+0"), TAUSTAR.ToString("0.00E+0"),
						                  besttau.ToString("0.00E+0"), iter - bestIter, iter - lastSignificantImprovementIter);
					}
					else
					{
						Console.Write ("best iter: {0} (SSE: {1}), current iter: {2} (SSE: {3}), tau*: {4}, tau: {5}, no imprvmnt since {6} iters.   \r",
						                  bestIter, bestSSE.ToString("0.00E+0"), iter, SSE[0].ToString("0.00E+0"), TAUSTAR.ToString("0.00E+0"),
						                  besttau.ToString("0.00E+0"), iter - bestIter);
					}
				}
				
				// HYBRID METHOD
				// check if switch to Gauss-Newton method is called for
				if (rootmethod == 2 && currentmethod == 0)	// if method is hybrid and current method is steepest descent
				{
					if (switchKeep <= iter - lastSignificantImprovementIter)
					{
						restoreBestEstimate();
						currentmethod = 1;
						Console.WriteLine("\n\n>>> switching to Gauss-Newton method...");
					}
				}
				
				// LEVENBERG-MARQUARDT METHOD
				// update mu
				if (currentmethod == 3)	    // if method is Levenberg-Marquardt
				{
					if (iter > bestIter)	// no improvement in this step
						mu = Math.Min(mu*muFactor, maxMu);
					else
						mu = Math.Max(mu/muFactor, minMu);
				}
			}
			
			restoreBestEstimate();
		}
		
		private static void restoreBestEstimate()
		{
			int j = 0;
			for (j=0; j<=g; j++) betavec[j, 0] = bestBetavec[j];
			k = 0;
			compute();		// compute phi, psi, etc for best estimate
		}
		
		private static void SteepestDescentStep()
		{
			// This routine computes a direction for the next step according to
			// steepest descent method. The negative of this step is stored in
			// betavec[.,1].
			
			ComputeGradient();
			
			// store (minus) direction of step in betavec[.,1]
			for (int j=1; j<=g; j++) betavec[j,1] = grad[j];
		}
		
		private static void GaussNewtonStep()
		{
			// This routine computes a direction for the next step according to
			// the Gauss-Newton method. Minus the Newton step is stored in betavec[.,1].
			
			ComputeJacobian();	    // compute the Jacobian of D...
			InvertJ();		        // ... and invert it
			
			// ... now, compute (invJ * D)  (store this is betavec[.,1])
			for (int j=1; j<=g; j++)
			{
				betavec[j,1] = 0;
				for (int jj=1; jj<=g; jj++)
				{
                    betavec[j,1] += (invJ[j, jj] * D[jj, 0]);
				}
			}
		}
			
		private static void LevenbergMarquardtStep()
		{
			// This routine computes a direction for the next step according to
			// Marquart method. The negative of this step is stored in
			// betavec[.,1].
			
			int j = 0; int jj = 0; int l = 0;	// indices
			
			double weightSD = mu/(1+mu);		// weight on steepest descent method
			double weightGN = 1/(1+mu);		    // weight on Gauss-Newton method
			
			// compute the gradient of SSE ...
			ComputeGradient();
			// ... and the Jacobian of D
			ComputeJacobian();
			
			// make copy of Jacobian (keepJ)
			// transpose Jacobian (TransJ)
			// make J = 0
			for (j=1; j<=g; j++)
			{
				for (jj=1; jj<=g; jj++)
				{
					keepJ[j,jj] = J[j,jj];
					TransJ[jj,j] = J[j,jj];
					J[j,jj] = 0;
				}
			}
			
			// compute Levenberg-Marquardt matrix, J' J + mu eye;
			// store result in J
			// [first perform J'J * weightGN]
			for (j=1; j<=g; j++)
			{
				for (jj=1; jj<=g; jj++)
				{
					for (l=1; l<=g; l++)
					{
						J[j,jj] += TransJ[j,l]*keepJ[l,jj];
					}
					J[j,jj] *= weightGN;
				}
			}
			// [then add weightSD*eye]
			for (j=1; j<=g; j++)
			{
				J[j,j] += weightSD;
			}
			
			// invert Levenberg-Marquardt matrix (result is in invJ)
			InvertJ();
			
			// compute invJ * grad; store in betavec[.,1]
			for (j=1; j<=g; j++)
			{
				betavec[j,1] = 0;
				for (l=1; l<=g; l++)
				{
					betavec[j,1] += invJ[j,l]*grad[l];
				}
			}
		}
		
		private static void ComputeGradient()
		{
			// Computes the gradient of SSE and stores it into the vector grad[].
			// Destroys betavec[.,1] and betavec[.,2] in the process.
			
			int j = 0;
			double v = 0;
			
			// compute gradient
    			for (j=1; j<=g; j++)
			{
				v = val(j);
				// copy current bid function into auxiliary vectors
				copyVectors(0,1);
				copyVectors(0,2);
				// decrease bid slightly
	    	    		k = 1;
				betavec[j, k] -= diffDelta;
				betavec[j, k] = Math.Max(betavec[j-1, k], betavec[j, k]);
				betavec[j, k] = Math.Min(betavec[j, k], v);
				compute();
				// increase bid slightly
				k = 2;
				betavec[j, k] += diffDelta;
				betavec[j, k] = Math.Max(betavec[j-1, k], betavec[j, k]);
				betavec[j, k] = Math.Min(betavec[j, k], v);
				compute();
				// compute derivative
				grad[j] = (SSE[2] - SSE[1]) / (betavec[j, 2] - betavec[j, 1]);
			}
		}
		
		private static void ComputeJacobian()
		{
			// Computes the Jacobian of D; stores result in matrix J.
			// Destroys betavec[.,1] and betavec[.,2] in the process.
			
			int j = 0; int jj = 0;		// three counters
			double v = 0;			    // auxiliary variable
			
    		for (j=1; j<=g; j++)
			{
				v = val(j);
				// copy current bid function into auxiliary vectors
				copyVectors(0,1);
				copyVectors(0,2);
				// decrease bid slightly
	    	    		k = 1;
				betavec[j, k] -= diffDelta;
				betavec[j, k] = Math.Max(betavec[j-1, k], betavec[j, k]);
				betavec[j, k] = Math.Min(betavec[j, k], v);
				compute();
				// increase bid slightly
				k = 2;
				betavec[j, k] += diffDelta;
				betavec[j, k] = Math.Max(betavec[j-1, k], betavec[j, k]);
				betavec[j, k] = Math.Min(betavec[j, k], v);
				compute();
				// fill in the components of the Jacobian
				for (jj=1; jj<=g; jj++)		// fill in column j
				J[jj,j] = (D[jj,2] - D[jj,1]) / (betavec[j, 2] - betavec[j, 1]);
			}
		}
		
		private static void InvertJ()
		{
			// This routine takes the gxg matrix stored in J and writes the inverse matrix
			// into invJ.  It destroys J in the process.  Also, the routine assumes that J
			// is invertible and has non-zero diagonal elements.
			
			int j = 0; int jj = 0; int l = 0;	// three counters
			double number = 0;			// aux variable used for matrix inversion
			
			// ... first make invJ = eye
			for (j=1; j<=g; j++)			// run through colimns
			{
				for (jj=1; jj<=g; jj++)		// run through rows
					invJ[jj,j] = 0;
			}
			for (j=1; j<=g; j++)			// fill diagonal
				invJ[j,j] = 1;
			
			// ... next, invert J
			for (j=1; j<=g; j++)
			{
				// [remove lower triangle]
				jj = 1;
        			while (jj < j)
        			{
	        	        	number = J[j, jj] / J[jj, jj];
			                for (l=1; l<=g; l++)
		                	{
	        	        		J[j, l] -= (number * J[jj, l]);
		        	        	invJ[j, l] -= (number * invJ[jj, l]);
		                	}
		        	        jj++;
        			}
				// [make unit diagonal components]
	            		number = J[j, j];
				for (jj=1; jj<=g; jj++)
				{
					J[j, jj] = J[j, jj] / number;
					invJ[j, jj] = invJ[j, jj] / number;
				}
			}
			// [remove upper triangle]
			for (j=1; j<=g-1; j++)
			{
	            jj = j + 1;
	            while (jj <= g)
	            {
	                number = J[j, jj] / J[jj, jj];
	                for (l=1; l<=g; l++)
	                {
	                    J[j, l] -= (number * J[jj, l]);
	                    invJ[j, l] -= (number * invJ[jj, l]);
	                }
	                jj++;
	            }
			}
		}
		
		// ****************************************
		// STEP-SIZE SEARCH
		// ****************************************
		
		private static void OptimizeTau_GoldenSection()
		{
			// This routine first determines the maximum stepsize TAUSTAR that still satisfies
			// weak monotonicity and weak bid shading. Then, it selects the optimal stepsize
			// 0 < tau < TAUSTAR that minimizes the SSE using golden section line search.
			
			// The routine expects -DeltaBeta (computed either with the steepest decent method
			// or with Newton's method) to be stored in betavec[.,1]. It overwrites betavec[.,0]
			// with the new estimate.
			
			int j = 0;				// two counters
			
			double a; double b;
			double tau1; double tau2;
			double SSE1; double SSE2;
			
			// find largest stepsize TAUBAR that still fulfills bid shading
			if (betavec[1,1] >= 0)
				taubar[1] = GRANDMAXTAU;		// no constraint
			else
				taubar[1] = -(val(1)-betavec[1,0]) / betavec[1,1];
			TAUBAR = taubar[1];
			for (j=2; j<=g; j++)
			{
				if (betavec[j,1] >= 0)
					taubar[j] = GRANDMAXTAU;	// no constraint
				else
					taubar[j] = -(val(j)-betavec[j,0]) / betavec[j,1];
				if (taubar[j] < TAUBAR)
					TAUBAR = taubar[j];
			}
			
			// find largest stepsize TAUTILDE that still fulfills weak monotonicity
			if (-betavec[1,1] >= 0)
				tautilde[1] = GRANDMAXTAU;		// no constraint
			else
				tautilde[1] = betavec[1,0] / betavec[1,1];
			TAUTILDE = tautilde[1];
			for (j=2; j<=g; j++)
			{
				if (-(betavec[j,1]-betavec[j-1,1]) >= 0)
					tautilde[j] = GRANDMAXTAU;	// no constraint
				else
					tautilde[j] = (betavec[j,0]-betavec[j-1,0]) / (betavec[j,1]-betavec[j-1,1]);
				if (tautilde[j] < TAUTILDE)
					TAUTILDE = tautilde[j];
			}
			
			// observe the stronger of the two constraints
			TAUSTAR = Math.Min(TAUTILDE,TAUBAR);
			
			// compute SSE for different step sizes tau
			// the bid functions for a given tau is stored in betavec[.,2]
			k = 2;
			// check if best step size is the smallest possible
			// but still leads to a deterioration of SSE;
			// if so, search for smaller step sizes.
			a = 0; b = 1;
			tau1 = 1-golden;
			for (j=1; j<=g; j++)
				betavec[j, k] = betavec[j, 0] - (tau1 * TAUSTAR * betavec[j,1]);
			compute();
			SSE1 = SSE[k];
			tau2 = golden;
			for (j=1; j<=g; j++)
				betavec[j, k] = betavec[j, 0] - (tau2 * TAUSTAR * betavec[j,1]);
			compute();
			SSE2 = SSE[k];
			while (tau2-tau1 > tauEps)
			{
				if (SSE1>SSE2)
				{
					a=tau1; tau1=tau2; SSE1=SSE2;
					tau2 = a + golden*(b-a);
					for (j=1; j<=g; j++)
						betavec[j, k] = betavec[j, 0] - (tau2 * TAUSTAR * betavec[j,1]);
					compute();
					SSE2 = SSE[k];
				}
				else
				{
					b=tau2; tau2=tau1; SSE2=SSE1;
					tau1 = a + (1-golden)*(b-a);
					for (j=1; j<=g; j++)
						betavec[j, k] = betavec[j, 0] - (tau1 * TAUSTAR * betavec[j,1]);
					compute();
					SSE1 = SSE[k];
				}
			}
			
			// compute next estimate of betavec
			besttau =  TAUSTAR * (tau1+tau2) / 2;
			for (j=1; j<=g; j++)
				betavec[j, 0] -= (besttau * betavec[j,1]);
			k = 0;
			compute();
		}
		
		private static void OptimizeTau_BruteForce()
		{
			// This routine first determines the maximum stepsize TAUSTAR that still satisfies
			// weak monotonicity and weak bid shading. Then, it selects the optimal stepsize
			// 0 < tau < TAUSTAR that minimizes the SSE.
			
			// The routine expects -DeltaBeta (computed either with the steepest decent method
			// or with Newton's method) to be stored in betavec[.,1]. It overwrites betavec[.,0]
			// with the new estimate.
			
			int j = 0; int jj = 0;				// two counters
			double keepSSE = SSE[0];			// needed to check whether there is an improvement
			double shrinkFactor = 1;			// shrinkTimes and shrinkFactor will change if
												//     minimum stepsize is still too large
			
			// find largest stepsize TAUBAR that still fulfills bid shading
			if (betavec[1,1] >= 0)
				taubar[1] = GRANDMAXTAU;		// no constraint
			else
				taubar[1] = -(val(1)-betavec[1,0]) / betavec[1,1];
			TAUBAR = taubar[1];
			for (j=2; j<=g; j++)
			{
				if (betavec[j,1] >= 0)
					taubar[j] = GRANDMAXTAU;	// no constraint
				else
					taubar[j] = -(val(j)-betavec[j,0]) / betavec[j,1];
				if (taubar[j] < TAUBAR)
					TAUBAR = taubar[j];
			}
			
			// find largest stepsize TAUTILDE that still fulfills weak monotonicity
			if (-betavec[1,1] >= 0)
				tautilde[1] = GRANDMAXTAU;		// no constraint
			else
				tautilde[1] = betavec[1,0] / betavec[1,1];
			TAUTILDE = tautilde[1];
			for (j=2; j<=g; j++)
			{
				if (-(betavec[j,1]-betavec[j-1,1]) >= 0)
					tautilde[j] = GRANDMAXTAU;	// no constraint
				else
					tautilde[j] = (betavec[j,0]-betavec[j-1,0]) / (betavec[j,1]-betavec[j-1,1]);
				if (tautilde[j] < TAUTILDE)
					TAUTILDE = tautilde[j];
			}
			
			// observe the stronger of the two constraints
			TAUSTAR = Math.Min(TAUTILDE,TAUBAR);
			
			// compute SSE for different step sizes tau
			// the bid functions for a given tau is stored in betavec[.,2]
			k = 2;
			// check if best step size is the smallest possible
			// but still leads to a deterioration of SSE;
			// if so, search for smaller step sizes.
			shrinkTimes = -1;
			do
			{
				shrinkTimes++;
				shrinkFactor = Math.Pow(tausteps,shrinkTimes);
				
				// compute SSE for different step sizes
				for (jj=1; jj<=tausteps-1; jj++)
				{
					tau = factor[jj] * TAUSTAR / shrinkFactor;
					for (j=1; j<=g; j++)
						betavec[j, 2] = betavec[j, 0] - (tau * betavec[j,1]);
					compute();
					SSEtau[jj] = SSE[k];
				}
				
				// search best step size
				bestFactor = 1;
				besttau = factor[1] * TAUSTAR / shrinkFactor;
				bestSSEtau = SSEtau[1];
				for (jj=2; jj<=tausteps-1; jj++)
				{
					if (SSEtau[jj] < bestSSEtau)
					{
						bestFactor = jj;
						besttau = factor[jj] * TAUSTAR / shrinkFactor;
						bestSSEtau = SSEtau[jj];
					}
				}
			} while (bestFactor == 1 && bestSSEtau > keepSSE &&
			    (TAUSTAR / shrinkFactor) / tausteps > bestSSE / (g+1));
			
			// compute next estimate of betavec
			for (j=1; j<=g; j++)
				betavec[j, 0] = betavec[j, 0] - (besttau * betavec[j,1]);
			k = 0;
			compute();
		}
		
		// ****************************************
		// IMPLEMENTATION OF THE MODEL
		// computation of D, phi, psi etc.
		// ****************************************
		
		private static void copyVectors(byte k1, byte k2)
		{
			// There are three copies of all the key vectors, k = 0,1,2.
			// This routine copies one into another.
			int j =0;		// index on the grid
			for (j=1; j<=g; j++)
			{
				betavec[j, k2] = betavec[j, k1];
				betaPrime[j, k2] = betaPrime[j, k1];
				psibarvec[j, k2] = psibarvec[j, k1];
				psibarPrime[j, k2] = psibarPrime[j, k1];
				phivec[j, k2] = phivec[j, k1];
				phiPrime[j, k2] = phiPrime[j, k1];
				D[j, k2] = D[j, k1];
			}
			SSE[k2] = SSE[k1];
		}
		
		private static void copyBetaVec(byte k1, byte k2)
		{
			// This routine copies just the betavec vector.
			int j = 0;		// index on the grid
			for (j=1; j<=g; j++) betavec[j, k2] = betavec[j, k1];
		}
		
		private static int idx(double x)
		{
			// The valuations are represented as discrete indices, running from 0 to g.
			// This functions maps a valuation into an index.
			int i = 0;		// the index we look for
			double d = (double) x*g;
			i = (int) Math.Ceiling(d);
			i = Math.Max(i, 0);
			i = Math.Min(i, g);
			return i;
		}
		
		private static double val(int i)
		{
			// The valuations are represented as discrete indices, running from 0 to g.
			// This functions maps an index into a valuation.
			double v = (double) i/g;
			return v;
		}
		
  		private static void compute()
		{
			// compute all vectors (in the right sequence)
			computeBetaPrime();
			computePsivec();
			computePsiPrime();
			computePhivec();
			computePhiPrime();
			computeDiffeq();
		}
		
		private static void computeBetaPrime()
		{
			// compute betaPrime from betavec
			int j = 0;		// index on the grid
			for (j=1; j<=g; j++)
				betaPrime[j, k] = (betavec[j, k] - betavec[j - 1, k]) * g;
		}
		
		private static double beta(double x)
		{
			// This function maps a valuation x into an approximate bid, using the
			// piecewise linear approximation. So this is not a map from
			// {0,...,g} -> R (like betavec), but from R -> R.
			int j = idx(x);
			double b = 0;
			if (j == 0) b = 0;
			else b = betavec[j-1, k] + betaPrime[j, k] * (x - val(j-1));
			return b;
		}
		
		private static double inverseBeta(double y)
		{
			// maps a bid y into its preimage under beta.
			int j = 0;		// index on the grid
			double v = 0;	// the valuation we look for
			if (y >= betavec[g, k])
			{
				v = 1;
			}
			else
			{
				if (y == 0)
				{
					v = 0;
				}
				else
				{
			        do j++; while (betavec[j, k] < y && j < g);
			        v = val(j-1) + (y - betavec[j - 1, k]) / betaPrime[j, k];
				}
			}
			return v;
		}
		
		private static int inverseBetaIdx(double y)
		{
			// maps a bid y into the index of its preimage under beta.
			int j = 0;		// the index we look for
			if (y >= betavec[g, k])
			{
				j = g;
			}
			else
			{
		        do j++; while (betavec[j, k] < y && j < g);
			}
			return j;
		}
		
		private static double FOC2(int i)
		{
			// This is the differential equation we try to solve, for n = 2.
			// We could use FOCn for arbitrary n, but the computations are much
			// simpler with n=2, and thus probably faster and maybe more precise.
			double v = val(i);
			double FOC;
			FOC = phiPrime[i, k] * (v - (1-alpha) * betavec[i, k]
									- alpha * beta(phivec[i, k]))
				- (1-alpha) * betaPrime[i, k] * phivec[i, k]
				+ alpha * (v - betavec[i, k]) * (psibarPrime[i, k] / 2 - 1)
				- (1-alpha) * (psibarvec[i, k] - v);
			return FOC;
		}
		
		private static double FOCn(int i)
		{
			// This is the differential equation we try to solve, for arbitrary n.
			double v = val(i);
			double FOC;
			FOC = phiPrime[i, k]
				  * (v - (1-alpha) * betavec[i, k] - alpha * beta(phivec[i, k]))
				  * (n-1) * Math.Pow(phivec[i, k], n-2)
				- (1-alpha) * betaPrime[i, k] * Math.Pow(phivec[i, k], n-1)
				+ alpha * (v - betavec[i, k]) * (psibarPrime[i, k] / 2 - 1)
				  * (n-1) * Math.Pow(v, n-2)
				- (1-alpha) * (psibarvec[i, k] - v) * (n-1) * Math.Pow(v, n-2)
				+ alpha*(n-1)*(n-2)*Math.Pow(v,n-3)
				  * (v*(psibarvec[i, k]-v) - integrateBeta(i,psibarvec[i, k]));
			return FOC;
		}
		
		private static void computeDiffeq()
		{
			// Fill the vector D with the values of the
			// differential equation on all the grid points.
		    int j = 0;
			SSE[k] = 0;
		    if (n == 2)
		    {
		        for (j=1; j<=g; j++)
		        {
		            D[j, k] = FOC2(j);
		        	SSE[k] += (D[j, k] * D[j, k]);
		        }
		    }
		    else
		    {
		        for (j=1; j<=g; j++)
		        {
		            D[j, k] = FOCn(j);
		        	SSE[k] += (D[j, k] * D[j, k]);
		        }
		    }
			SSE[k] = SSE[k]/2;
		}
		
		private static void computePsivec()
		{
			// compute psibarvec from betavec
			int j = 0;			// index on the grid
			psibarvec[0, k] = 0;
			for (j=1; j<=g; j++)
			{
				psibarvec[j, k] = inverseBeta((val(j) + betavec[j, k]) / 2);
				psibarvec[j, k] = Math.Max(psibarvec[j, k],val(j));
			}
		}
		
		private static void computePsiPrime()
		{
			// compute psibarPrime from betavec and betaPrime
			int j = 0;			// index on the grid
			for (j=1; j<=g; j++)
			{
				if (psibarvec[j, k] >= 1)
				{
					psibarPrime[j, k] = 0;
				}
				else
				{
					psibarPrime[j, k] = (1 + betaPrime[j, k]) / 
						(2 * betaPrime[inverseBetaIdx((val(j) + betavec[j, k]) / 2), k]);
				}
			}
		}
		
		private static void computePhivec()
		{
			// compute phivec from betavec.
			// Note that phi is the solution of a fixed point problem.
			int j;			    // index on the grid
			double phiEq;		// equation we search the fixed point of
			double phiHi;		// barriers for bisection method
			double phiLo;
			double phiMid;
			double span = 0;	// difference between phiHi and phiLo
						        // procedure stops when computer can no longer
						        // compute smaller differences.
			
			// We compute phi with the bisection method.
			// phi is the root of the equation
			//   phiEq := phi + beta(phi) - 2beta.
			// phiEq is monotonic in phi.
			// If phi = 0, then phiEq = -2beta < 0.
			// If phi = x, then phiEq = x+beta(x)-2beta(x) > 0 (if shading).
			phivec[0, k] = 0;
			for (j=1; j<=g; j++)
			{
				phiHi = val(j);
				phiLo = 0;
				if (betavec[j, k] >= val(j))
				{
					phivec[j, k] = val(j);
				}
				else
				{
					do
					{
						span = phiHi-phiLo;
						phiMid = (phiLo + phiHi) / 2;
						phiEq = phiMid + beta(phiMid) - 2 * betavec[j, k];
						if (phiEq < 0) phiLo = phiMid;
						else           phiHi = phiMid;
					} while (phiHi-phiLo < span);
					// continue until machine cannot further decrease the difference
					phivec[j, k] = phiMid;
				}
			}
		}
		
		private static void computePhiPrime()
		{
			// compute phiPrime from betaPrime and phivec
			int j = 0;			// index on the grid
			for (j=1; j<=g; j++)
				phiPrime[j, k] = (2 * betaPrime[j, k]) / (1 + betaPrime[idx(phivec[j, k]), k]);
		}
		
		private static double integrateBeta(int lowidx, double high)
		{
			// compute numerical integral of beta from low to high
			int highidx = idx(high) - 1;
			double integral = 0;			// store result
			int j = 0;						// counter
			
			if (lowidx < highidx)
			{
				for (j=lowidx; j<highidx; j++)
				{
					integral += (betavec[j, k] + betavec[j+1, k]);
				}
				integral = integral / (2 * g);
				integral += ((high-val(highidx)) * (beta(high)+betavec[highidx, k]) / 2);
			}
			else
			{
				integral = (high-val(lowidx)) * (beta(high)+betavec[lowidx,k])/2;
			}
			
			return integral;
		}
		
		// ****************************************
		// EFFICIENCY AND DISTRIBUTION
		// ****************************************
		
		private static double integrateZ(double low, double high)
		{
			// numerically integrates a piecewise linear function
			// expects the function to be stored in Zvec
			int lowidx = idx(low);
			int highidx = idx(high) - 1;
			double integral = 0;	// store result
			int j = 0;			    // counter
			
			if (lowidx > highidx)
			{
				integral = (high-low) * (Z(high)+Z(low))/2;
			}
			
			if (lowidx == highidx)
			{
				integral = (val(lowidx)-low) * (Zvec[lowidx]+Z(low))/2
					+ (high-val(highidx)) * (Z(high)+Zvec[highidx])/2;
			}
			
			if (lowidx < highidx)
			{
				for (j=lowidx; j<highidx; j++)
				{
					integral += ((Zvec[j] + Zvec[j+1]) / 2);
				}
				integral = integral/(highidx-lowidx) * (val(highidx)-val(lowidx));
				integral = integral + (val(lowidx)-low) * (Zvec[lowidx]+Z(low))/2 +
					(high-val(highidx)) * (Z(high)+Zvec[highidx])/2;
			}
			
			return integral;
		}
		
		private static double Z(double x)
		{
			// This function maps a x into Z(x), where Z is a piecewise linear function
			// defined by Zvec.

			int j = idx(x);
			double b = 0;
			if (j == 0) b = 0;
			else b = Zvec[j-1] + (Zvec[j] - Zvec[j-1]) * (x - val(j-1)) / g;
			return b;
		}
		
		private static void ComputeH()
		{
			int j = 0;
			
			// compute winning probability of bidder with valuation val(j)
			// \int_0^x H(y) dy
			H[0] = 0;
			for(j=1; j<=g; j++)
				H[j] = Math.Pow(psibarvec[j,0],n-1) - Math.Pow(val(j),n-1) + Math.Pow(phivec[j,0],n-1);
			
			// integrate H
			IntH[0] = 0;
			for(j=1; j<=g; j++)
				IntH[j] = IntH[j-1] + (H[j]+H[j-1])/(2*g);
		}
		
		private static void ComputePayoffs()
		{
			int j = 0; int jj = 0;			// index on the grid
			double[] IntZ = new double[g+1];	// integral
			
			double betax = 0;
			
			k=0;		// work with copy-0 of the vectors
			
			// welfare loss
			IntZ[0] = 0;
			for (jj=1; jj<=g; jj++)
			{
				// \int_0^\phi(x) (x-y) y^{n-2} dy
				Zvec[0] = 0;
				for (j=1; j<=g; j++)
				{
					Zvec[j] = (val(jj)-val(j)) * Math.Pow(val(j),n-2);
				}
				IntZ[jj] = integrateZ(phivec[jj,0],val(jj));
			}
			// \int_0^1 ... dx
			Zvec[0] = 0;
			for (jj=1; jj<=g; jj++)
			{
				Zvec[jj] = IntZ[jj];
			}
			WelfareLoss = (double) n * (n-1) * integrateZ(0,1);
			
			WelfareEfficient = (double) n/(n+1);
			Welfare = WelfareEfficient - WelfareLoss;
			
			// auctioneer's payoff
			IntZ[0] = 0;
			for (jj=1; jj<=g; jj++)
			{
				betax = betavec[jj,0];
				// \int_0^\phi(x) (\beta(x)-\beta(y)) y^{n-2} dy
				Zvec[0] = 0;
				for (j=1; j<=g; j++)
				{
					Zvec[j] = (betax-betavec[j,0]) * Math.Pow(val(j),n-2);
				}
				IntZ[jj] = integrateZ(0,phivec[jj,0]);
				// \int_\phi(x)^x (y-\beta(x)) y^{n-2} dy
				Zvec[0] = 0;
				for (j=1; j<=g; j++)
				{
					Zvec[j] = (val(j)-betax) * Math.Pow(val(j),n-2);
				}
				IntZ[jj] += integrateZ(phivec[jj,0],val(jj));
			}
			// \int_0^1 ... dx
			Zvec[0] = 0;
			for (jj=1; jj<=g; jj++)
			{
				Zvec[jj] = IntZ[jj];
			}
			piAuctioneer = (double) n * (n-1) * (1-alpha) * integrateZ(0,1);
			
			// seller's payoff
			IntZ[0] = 0;
			for (jj=1; jj<=g; jj++)
			{
				betax = betavec[jj,0];
				// \int_0^\phi(x) \beta(y) y^{n-2} dy
				Zvec[0] = 0;
				for (j=1; j<=g; j++)
				{
					Zvec[j] = betavec[j,0] * Math.Pow(val(j),n-2);
				}
				IntZ[jj] = integrateZ(0,phivec[jj,0]);
				// \int_\phi(x)^x (y-\beta(x)) y^{n-2} dy
				Zvec[0] = 0;
				for (j=1; j<=g; j++)
				{
					Zvec[j] = Math.Pow(val(j),n-2);
				}
				IntZ[jj] += betax * integrateZ(phivec[jj,0],val(jj));
			}
			// \int_0^1 ... dx
			Zvec[0] = 0;
			for (jj=1; jj<=g; jj++)
			{
				Zvec[jj] = IntZ[jj];
			}
			piSeller = (double) n * (n-1) * integrateZ(0,1);
			
			// sum of bidders' ex ante expected payoff (before knowing their valuation)
			// \int_0^1 \int_0^x H(y) dy dx
			Zvec[0] = 0;
			for (jj=1; jj<=g; jj++)
			{
				Zvec[jj] = IntH[jj];
			}
			piAllBidders = (double) n * integrateZ(0,1);
		}

        // ****************************************
        // TEST FOR GLOBAL OPTIMALITY
        // ****************************************

        private static void ComputeUvec()
        {
            Console.WriteLine(">>> Computing payoff function U(v,x)...\n");
            int vidx = 0; int xidx = 0;
            double v = 0; double x = 0;
            // perform computation of off-equilibrium payoff function
            for (vidx = 0; vidx <= g; vidx++)
            {
                v = val(vidx);
                for (xidx = 0; xidx <= g; xidx++)
                {
                    // compute U(v,x)
                    x = val(xidx);
                    if (n == 2)
                    {
                        ZisBeta();
                        Uvec[vidx,xidx] = phivec[xidx,k] * (v-(1-alpha)*betavec[xidx,k]) +
                            (psivec[vidx,xidx]-x) * (v-(1-alpha)*x);
                        Uvec[vidx,xidx] += -alpha * (integrateZ(0,phivec[xidx,k]));
                        if (psivec[vidx,xidx] > x)
                            Uvec[vidx,xidx] += -alpha * integrateZ(x,psivec[vidx,xidx]);
                    }
                    else
                    {
                        Uvec[vidx,xidx] = Math.Pow(phivec[xidx,k],n-1) * (v-(1-alpha)*betavec[xidx,k]) +
                            (psivec[vidx,xidx] - x) * (n-1)*Math.Pow(x,n-2) * (v-(1-alpha)*x);
                        ZisYN2Beta();
                        Uvec[vidx,xidx] += -alpha*(n-1) * (integrateZ(0,phivec[xidx,k]));
                        if (psivec[vidx,xidx] > x)
                        {
                            ZisBeta();
                            Uvec[vidx,xidx] += -alpha * (n-1) * Math.Pow(x,n-2) * integrateZ(x,psivec[vidx,xidx]);
                        }
                    }
                    // keep track of maximum and maximand
                    if (Uvec[vidx, xidx] > themax[vidx])
                    {
                        themax[vidx] = Uvec[vidx, xidx];
                        argmax[vidx] = x;
                    }
                }
            }
        }

        private static void ZisBeta()
        {
            // let Zvec = betavec (for numerical integration)
            for (int h = 0; h <= g; h++)
                Zvec[h] = betavec[h,k];
        }

        private static void ZisYN2Beta()
        {
            // let Zvec[y] = y^(n-2) * betavec[y] (for numerical integration)
            for (int h = 0; h <= g; h++)
                Zvec[h] = Math.Pow(val(h),n-2) * betavec[h,k];
        }

        private static void ComputePsitilde()
        {
            int vidx = 0; int xidx = 0;
            double v = 0; double x = 0;
            for (vidx = 0; vidx <= g; vidx++)
            {
                v = val(vidx);
                for (xidx = 0; xidx <= g; xidx++)
                {
                    x = val(xidx);
                    psitildevec[vidx, xidx] = inverseBeta((v - (1 - alpha) * x) / alpha);   // acceptance condition
                    psivec[vidx, xidx] = Math.Max(x, Math.Min(psitildevec[vidx, xidx], psibarvec[xidx, k]));
                }
            }
        }

        // ****************************************
		// OUTPUT ROUTINES
		// ****************************************
		
		private static void OutputParams()
		{
			// report the parameters of the current computation
			Console.WriteLine ("COMPUTATION OF BID FUNCTION for FIRST-PRICE AUCTION");
			Console.WriteLine ("with TYPE I and TYPE II CORRUPTION");
			Console.WriteLine ("----------------------------------------------------------------------------------------------------------------------\n");
			Console.WriteLine (" PARAMETERS OF THE AUCTION GAME");
			Console.WriteLine ("                            # of bidders, n = {0}", n);
			Console.WriteLine ("              winning bidder's share, alpha = {0}\n", alpha);
			Console.WriteLine (" DISCRETISATION OF VALUATIONS");
			if (NbSubdivisions > 0)
			{
				Console.WriteLine ("            initial grid on valuation space = {0}", g);
				Console.WriteLine ("                         subdivide {0} times by {1} ", NbSubdivisions, SubdivisionFactor);
				Console.WriteLine ("           ultimate grid on valuation space = {0}\n", max_g);
			}
			else
			{
				Console.WriteLine ("                 grid on valuation space, g = {0}\n", max_g);
			}
			if (initmethod == 0)
			{
				Console.WriteLine (" START FROM LINEAR INITIAL BID FUNCTION");
				Console.WriteLine ("                              initial slope = {0}\n", slope.ToString("0.000"));
			}
			else
			{
				Console.WriteLine (" PARAMETER FOR THE GRID SEARCH");
				Console.WriteLine ("                        steps per dimension = {0}\n", GridSearchSteps);
			}
			Console.Write (" ROOT FINDING: ");
			switch (rootmethod)
			{
				case 0:
					Console.WriteLine ("using steepest descent method\n");
					break;
				case 1:
					Console.WriteLine ("using Gauss-Newton method (inverse Jacobian)\n");
					break;
				case 2:
					Console.WriteLine ("using hybrid method");
					Console.WriteLine ("   Start with steepest descent method. Switch to Gauss-Newton\n" +
					                   "   method if improvement is less than {0} over {1} consecutive\n" +
					                   "   iterations.\n", switchEps, switchKeep);
					break;
				case 3:
					Console.WriteLine ("using Levenberg-Marquardt method");
					Console.WriteLine ("                                 initial mu = {0}", initMu.ToString("0E+0"));
					Console.WriteLine ("                                  mu factor = {0}", muFactor.ToString());
					Console.WriteLine ("                                 maximum mu = {0}", maxMu.ToString("0E+0"));
					Console.WriteLine ("                                 minimum mu = {0}", minMu.ToString("0E+0"));
					break;
			}
			Console.WriteLine (" STOPPING CONDITIONS");
			Console.WriteLine ("                                    epsilon = {0}", eps.ToString("0E+0"));
			Console.WriteLine ("                             keep iterating = {0}", keepiter);
			Console.WriteLine ("                             max iterations = {0}\n", maxiter);
			Console.Write (" STEP-SIZE OPTIMIZATION: ");
			if (taumethod == 0)
			{
				Console.WriteLine ("brute force");
				Console.WriteLine ("                       search steps for tau = {0}", tausteps);
			}
			else
			{
				Console.WriteLine ("golden section search");
				Console.WriteLine ("                                tau epsilon = {0}", tauEps.ToString("0E+0"));
			}
			Console.WriteLine ("                        upper bound for tau = {0}\n", GRANDMAXTAU.ToString());
			Console.WriteLine (" FINITE DIFFERENCE PARAMETER");
			Console.WriteLine ("                                 diff delta = {0}\n", diffDelta.ToString("0E+0"));
			Console.WriteLine ("======================================================================================================================\n");
		}
		
		private static void OutputResult()
		{
			// This routine outputs the final result.
			int j = 0;				// index on the grid
			double maxabserr = 0;	// maximum absolute deviation of the differential equation from zero
			double meanerr = 0;		// average deviation of the differential equation from zero
			string item = "";		// auxiliary string variable used to format some strings for output
			
			compute();				// compute phi, psi, D, SSE
			
			// compute max abs error and mean error
	        for (j=1; j<=g; j++)
	        {
	        	maxabserr = Math.Max(maxabserr,Math.Abs(D[j, k]));
	        	meanerr += D[j, k];
	        }
	        meanerr = meanerr / (g+1);
			
			// report summary statistics for the best estimate
			item = "             SSE: " + SSE[k].ToString("E") + "\n"
			     + "            RMSE: " + Math.Sqrt(2*SSE[k]/(g+1)).ToString("E") + "\n"
			     + " max absol error: " + maxabserr.ToString("E") + "\n"
			     + "      mean error: " + meanerr.ToString("E") + "\n";
			Console.WriteLine ("\n{0}", item);
			
			// output the bid function (best estimate)
			Console.WriteLine ("----------------------------------------------------------------------------------------------------------------------");
			Console.WriteLine (" v                       beta                    phi                     psi                     D                    ");
			Console.WriteLine ("----------------------------------------------------------------------------------------------------------------------");
			Console.WriteLine (" 0                       0                       0                       0                       +0.000000000E+000    ");
			for (j=1; j<=g; j++)
			{
				item = " " + val(j).ToString(); Console.Write(item.PadRight(24, (char)32));
				item = " " + betavec[j,k].ToString(); Console.Write(item.PadRight(24, (char)32));
				item = " " + phivec[j,k].ToString(); Console.Write(item.PadRight(24, (char)32));
				item = " " + psibarvec[j,k].ToString(); Console.Write(item.PadRight(24, (char)32));
				item = " " + D[j,k].ToString("+0.000000000E+000;-0.000000000E+000") + " "; Console.WriteLine(item.PadRight(24, (char)32));
			}
			Console.WriteLine("----------------------------------------------------------------------------------------------------------------------\n");
		}
		
		private static void OutputWelfareResults()
		{
			// This routine outputs the results concerning the welfare implications
			// and the distribution of payoffs.
			int j = 0;				// index on the grid
			string item = "";		// auxiliary string variable used to format some strings for output
			
			double piSellerEfficient = (double) (n-1)/(n+1);
			double piSellerChange = piSeller - piSellerEfficient;
			double piSellerRelativeChange = (piSellerChange/piSellerEfficient) * 100;

			double piAllBiddersEfficient = (double) 1/(n+1);
			double piAllBiddersChange = piAllBidders - piAllBiddersEfficient;
			double piAllBiddersRelativeChange = (piAllBiddersChange/piAllBiddersEfficient) * 100;
			
			double WelfareChange = -WelfareLoss;
			double WelfareRelativeChange = (WelfareChange/WelfareEfficient) * 100;
			
//			AllBiddersAsResidual = Welfare - piSeller - piAuctioneer;
			
			Console.WriteLine ("======================================================================================================================\n");
			Console.WriteLine ("Welfare and distribution...\n");
			
			// output expected payoffs of auctioneer and of seller, and ex ante welfare loss
			Console.WriteLine ("                   expected value of this auction: {0}", Welfare.ToString("0.000"));
			Console.WriteLine ("                      ... in an efficient auction: {0}", WelfareEfficient.ToString("0.000"));
			Console.WriteLine ("                                  absolute change: {0}", WelfareChange.ToString("+0.000;-0.000"));
			Console.WriteLine ("                                  relative change: {0}%\n", WelfareRelativeChange.ToString("+0.0;-0.0"));

			Console.WriteLine ("         seller's expected payoff in this auction: {0}", piSeller.ToString("0.000"));
			Console.WriteLine ("                      ... in an efficient auction: {0}", piSellerEfficient.ToString("0.000"));
			Console.WriteLine ("                                  absolute change: {0}", piSellerChange.ToString("+0.000;-0.000"));
			Console.WriteLine ("                                  relative change: {0}%\n", piSellerRelativeChange.ToString("+0.0;-0.0"));
			
			Console.WriteLine ("                     auctioneer's expected payoff: {0}\n", piAuctioneer.ToString("0.000"));

			Console.WriteLine ("                 sum of bidders' expected payoffs: {0}", piAllBidders.ToString("0.000"));
			Console.WriteLine ("                      ... in an efficient auction: {0}", piAllBiddersEfficient.ToString("0.000"));
			Console.WriteLine ("                                  absolute change: {0}", piAllBiddersChange.ToString("+0.000;-0.000"));
			Console.WriteLine ("                                  relative change: {0}%\n", piAllBiddersRelativeChange.ToString("+0.0;-0.0"));

			Console.WriteLine ("         check (should be zero, within precision): {0}\n", (piAllBidders + piSeller + piAuctioneer - Welfare).ToString("0.00E0"));

			// output winning probabilities and expected payoffs of bidders
			Console.WriteLine ("bidders' winning probabilities and expected payoffs");
			Console.WriteLine ("----------------------------------------------------------------------------------------------------------------------");
			Console.WriteLine (" v                       H                       G                       int H                   int G                ");
			Console.WriteLine ("----------------------------------------------------------------------------------------------------------------------");
			Console.WriteLine (" 0                       0                       0                       0                       0                    ");
			for (j=1; j<=g; j++)
			{
				item = " " + val(j).ToString(); Console.Write(item.PadRight(24, (char)32));
				item = " " + H[j].ToString(); Console.Write(item.PadRight(24, (char)32));
				item = " " + Math.Pow(val(j),n-1).ToString(); Console.Write(item.PadRight(24, (char)32));
				item = " " + IntH[j].ToString(); Console.Write(item.PadRight(24, (char)32));
				item = " " + (Math.Pow(val(j),n)/n).ToString() + " "; Console.WriteLine(item.PadRight(24, (char)32));
			}
			Console.WriteLine("----------------------------------------------------------------------------------------------------------------------\n");
        }

        private static void OutputEquilibriumTest()
        {
            string item = "";		// auxiliary string variable used to format some strings for output
            Console.WriteLine("===========================================================================================================================\n");
            Console.WriteLine("test equilibrium condition");
            Console.WriteLine("---------------------------------------------------------------------------------------------------------------------------");
            Console.WriteLine(" v                       argmax                  themax                  U(v,v)                   intH                     ");
            Console.WriteLine("---------------------------------------------------------------------------------------------------------------------------");
            for (int j = 0; j <= g; j++)
            {
                item = " " + val(j).ToString(); Console.Write(item.PadRight(24, (char)32));
                item = " " + argmax[j].ToString(); Console.Write(item.PadRight(24, (char)32));
                item = " " + themax[j].ToString(); Console.Write(item.PadRight(24, (char)32));
                item = " " + Uvec[j, j].ToString(); Console.Write(item.PadRight(24, (char)32));
                item = " " + IntH[j].ToString() + " "; Console.WriteLine(item.PadRight(24, (char)32));
            }
            Console.WriteLine("---------------------------------------------------------------------------------------------------------------------------");
            double MaxMaximorum = 0; double ArgmaxMaximorum = 0;
            for (int j = 0; j <= g; j++)
            {
                MaxMaximorum = (double)Math.Max(MaxMaximorum,themax[j]-Uvec[j,j]);
                ArgmaxMaximorum = (double)Math.Max(ArgmaxMaximorum, Math.Abs(val(j) - argmax[j]));
            }
            Console.WriteLine(" max U(v,x*)-U(v,v) = " + MaxMaximorum.ToString());
            Console.WriteLine("         max |x*-v| = " + ArgmaxMaximorum.ToString());
            Console.WriteLine("---------------------------------------------------------------------------------------------------------------------------");
        }

        private static void OutputPsiAndU(double v)
        {
            // This routine outputs the results concerning the check for global optimality
            // of the solution to the first-order condition, i.e. proposal/acceptance/rejection
            // ranges and off-equilibrium payoff function.
            int vidx = idx(v);
            string item = "";		// auxiliary string variable used to format some strings for output
            Console.WriteLine("---------------------------------------------------------------------------------------------------------------------------");
            Console.WriteLine("regions and payoff function for v = {0}", v.ToString());
            Console.WriteLine(" x                       psibar                  psitilde                psi                     U                         ");
            Console.WriteLine("---------------------------------------------------------------------------------------------------------------------------");
            for (int j = 0; j <= g; j++)
            {
                item = " " + val(j).ToString(); Console.Write(item.PadRight(24, (char)32));
                item = " " + psibarvec[j, k].ToString(); Console.Write(item.PadRight(24, (char)32));
                item = " " + psitildevec[vidx, j].ToString(); Console.Write(item.PadRight(24, (char)32));
                item = " " + psivec[vidx, j].ToString(); Console.Write(item.PadRight(24, (char)32));
                item = " " + Uvec[vidx, j].ToString() + " "; Console.WriteLine(item.PadRight(24, (char)32));
            }
            Console.WriteLine("---------------------------------------------------------------------------------------------------------------------------\n");
        }

    }
}
